<?php
/**
 * 支持中国320个城市的天气预报控件
 * @author lonestone@qq.com
 *
 */
class WeatherWidget extends CWidget
{
    /**
     * 城市名称
     * @var string
     */
    public $location;
    
    private $cities;
    
    /**
     * 是否根据IP自动判断城市
     * @var boolean
     */
    public $autoDetect = true;
    
    /**
     * 是否缓存天气数据
     * @var boolean
     */
    public $cache = true;
    
    /**
     * 缓存天气数据的秒数
     * @var integer
     */
    public $cacheDuration = 3600;
    
    public function init() 
	{
	    $filename = Yii::app()->basePath . '/data/weather_cities.php';
	    if(!file_exists($filename)) @touch($filename);
	    
	    $this->cities = include($filename);
	    if($this->cities === false || !is_array($this->cities))
	    {
    	    //得到省代码
            $html = $this->getpage('http://service.weather.com.cn/plugin/data/city.xml');
            $provinces = array();
            $arr = explode(',', $html);
            foreach($arr as $str)
            {
                $data = explode('|', $str);
                $provinces[$data[1]] = $data[0];
            }
            //得到市代码
            $this->cities = array();
            foreach($provinces as $pro)
            {
                $html = $this->getpage('http://service.weather.com.cn/plugin/data/city'.$pro.'.xml');
                $arr = explode(',', $html);
                foreach($arr as $str)
                {
                    $data = explode('|', $str);
                    //得到页面代码
                    $html = $this->getpage('http://service.weather.com.cn/plugin/data/city'.$data[0].'01.xml');
                    $tmp = explode('|', $html);
                    if(isset($tmp[1])) $this->cities[$data[1]] = $tmp[1];
                }
            }
            @file_put_contents($filename, '<?php return '.var_export($this->cities, true).';');
	    }
	    if($this->autoDetect)
	    {
	        $client_ip = Yii::app()->request->userHostAddress;
            if($client_ip == '127.0.0.1') $client_ip = '202.103.24.68';//武汉
            $ipl = new IpLocation();
            $location = $ipl->getlocation($client_ip);
            if(isset($location['country']))
            {
                $country = mb_convert_encoding($location['country'],'utf-8','GBK');
                foreach ($this->cities as $name=>$value)
                {
                    if(strpos($country, $name) !== false )
                    {
                        $this->location = $name;
                    }
                }
            }
	    }
    }
    
    public function run() 
	{
        if(isset($this->cities[$this->location]))
        {
            $key = 'weather_data_'.$this->location;
            $data = Yii::app()->cache->get($key);  
            if($data === false)
            { 
                $html = $this->getpage("http://m.weather.com.cn/data/{$this->cities[$this->location]}.html");
                $data = json_decode($html);
                Yii::app()->cache->set($key, $data, 3600);
            }
            
            $this->render('index',array('data'=>$data));
        }
    }
    
    protected function getpage($url)
    {
        if(!strpos($url, '://')) return 'Invalid URI';
        $content = '';
        if(function_exists('curl_init'))
        {
            $handle = curl_init();
            curl_setopt($handle, CURLOPT_URL, $url);
            curl_setopt($handle, CURLOPT_CONNECTTIMEOUT, 20);
            curl_setopt($handle, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($handle, CURLOPT_FOLLOWLOCATION, 0);
            $content = curl_exec($handle);
            curl_close($handle);
        }
        elseif(function_exists('fsockopen'))
        {
            $urlinfo = parse_url($url);
            $host = $urlinfo['host'];
            $str = explode($host, $url);
            $uri = $str[1];
            unset($urlinfo, $str);
            $content = '';
            $fp = fsockopen($host, 80, $errno, $errstr, 30);
            if(!$fp)
            {
                $content = 'Can Not Open Socket...';
            }
            else
            {
                stream_set_timeout( $fp , 10 ) ; //超时时间

                $out = "GET ".$uri."/  HTTP/1.1\r\n";
                $out.= "Host: $host \r\n";
                $out.= "Accept: */*\r\n";
                $out.= "User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.1)\r\n)";
                $out.= "Connection: Keep-Alive\r\n\r\n";
                fputs($fp, $out);
                while (!feof($fp))
                {
                    $status = stream_get_meta_data( $fp ) ;
                    //读取数据超时
                    if( $status['timed_out'] ) break;
                    $content .= fgets($fp, 4069);
                }
                fclose($fp);
            }
        }
        if(empty($content)) $content = false;
        return $content;
    }
}